@@ -1,5 +1,7 @@
 #!/usr/bin/env python3
 
+'''This module contains AEDCompare class'''
+
 import logging
 import numpy as np
 import pandas as pd
@@ -10,6 +12,15 @@ from ingenannot.commands.command import Command
 
 
 class AEDCompare(Command):
+    '''
+    The AEDCompare is a Command running AED comparison.
+
+    Attributes:
+    -----------
+    fof : list of files to compare
+    statistics : perfom statistics
+    ncol : number of columns in graph legend
+    '''
 
     def __init__(self, args):
 
@@ -28,7 +39,7 @@ class AEDCompare(Command):
         Graphics.plot_cumulative_aed(sources, genes, "best","cumulative_best_AED.png",self.ncol)
 
         if self.statistics:
-            print(self.perform_stats(sources, genes).to_string())
+            print(self.perform_stats(sources, genes).to_csv(sep="\t"))
 
         return 0
 
@@ -37,7 +48,10 @@ class AEDCompare(Command):
 
         logging.info("Running statistics")
 
-        dfstats = pd.DataFrame(columns=['mean (tr)','median (tr)','stdev (tr)','mean (pr)','median (pr)','stdev (pr)', 'median_geom_x','median_geom_y','mean_distance', 'mean_distance_to_0','median_distance', 'median_distance_to_0'],index=sources, dtype='float64')
+        dfstats = pd.DataFrame(columns=['mean (tr)','median (tr)','stdev (tr)',
+            'mean (pr)','median (pr)','stdev (pr)', 'median_geom_x','median_geom_y',
+            'mean_distance', 'mean_distance_to_0','median_distance',
+            'median_distance_to_0'],index=sources, dtype='float64')
         for src in sources:
             # extract data
             aed_scores_tmp = []
@@ -61,10 +75,13 @@ class AEDCompare(Command):
             geom_median = Statistics.geometric_median(aed_scores)
             dfstats['median_geom_x'][src] = round(geom_median[0],3)
             dfstats['median_geom_y'][src] = round(geom_median[1],3)
-            dfstats['mean_distance'][src] = round(Statistics.mean_median_distance(aed_scores, geom_median)[0],3) 
-            dfstats['mean_distance_to_0'][src] = round(Statistics.mean_median_distance(np.array([[0],[0]]), geom_median)[0],3) 
-            dfstats['median_distance'][src] = round(Statistics.mean_median_distance(aed_scores, geom_median)[1],3) 
-            dfstats['median_distance_to_0'][src] = round(Statistics.mean_median_distance(np.array([[0],[0]]), geom_median)[1],3) 
-
+            dfstats['mean_distance'][src] = round(Statistics.mean_median_distance(aed_scores,
+                geom_median)[0],3)
+            dfstats['mean_distance_to_0'][src] = round(Statistics.mean_median_distance(
+                np.array([[0],[0]]), geom_median)[0],3)
+            dfstats['median_distance'][src] = round(Statistics.mean_median_distance(aed_scores,
+                geom_median)[1],3)
+            dfstats['median_distance_to_0'][src] = round(Statistics.mean_median_distance(
+                np.array([[0],[0]]), geom_median)[1],3)
 
         return dfstats
