@@ -43,23 +43,23 @@ class ArgParse():
     def load_all_subparsers(self):
         """Load all subparsers"""
 
-        self.subparser_validate()
-        self.subparser_compare()
-        self.subparser_select()
-        self.subparser_aed_strand_annotation_filter()
-        self.subparser_aed()
-        self.subparser_isoform_ranking() # coverage test
-        self.subparser_filter() # code
-        self.subparser_rescue_effectors() # code
-        self.subparser_effector_predictor() # code
-        self.subparser_utr_refine()
         self.subparser_add_sqanti3_isoforms()
+        self.subparser_aed()
         self.subparser_aed_compare() # code and test to do and validate
-        self.subparser_exonerate_to_gff()
-        self.subparser_rename()
+        self.subparser_aed_strand_annotation_filter()
         self.subparser_clusterize()
+        self.subparser_compare()
+        self.subparser_curation()
+        self.subparser_effector_predictor() # code
+        self.subparser_exonerate_to_gff()
+        self.subparser_filter() # code
+        self.subparser_isoform_ranking() # coverage test
+        self.subparser_rename()
+        self.subparser_rescue_effectors() # code
+        self.subparser_select()
         self.subparser_soclassif()
-
+        self.subparser_utr_refine()
+        self.subparser_validate()
 
 
     def subparser_soclassif(self):
@@ -67,7 +67,7 @@ class ArgParse():
 
         sbp = self.subparsers.add_parser(
                 'soclassif',
-                help='soclassif help'
+                help='Classify transcripts based on SO classification'
         )
         sbp.add_argument(
                 "fof",
@@ -103,7 +103,7 @@ class ArgParse():
 
         sbp = self.subparsers.add_parser(
                 'add_sqanti3_isoforms',
-                help='add_sqanti3_isoforms help'
+                help='Add Isoforms from sqanti3 outputs'
         )
         sbp.add_argument(
                 "Gff_genes",
@@ -142,7 +142,7 @@ class ArgParse():
 
         sbp = self.subparsers.add_parser(
                 'rescue_effectors',
-                help='rescue_effectors help'
+                help='Perform rescue of effector genes'
         )
         sbp.add_argument(
                 "Gff_genes",
@@ -162,29 +162,29 @@ class ArgParse():
         )
         sbp.add_argument(
                 "--signalp",
-                help="Path to signalp, default={} (from system lookup)"\
-                     .format(shutil.which('signalp')),
+                help=f"Path to signalp, default={shutil.which('signalp')} \
+                        (from system lookup)",
                 type=str,
                 default=shutil.which('signalp')
         )
         sbp.add_argument(
                 "--tmhmm",
-                help="Path to tmhmm, default={} (from system lookup)"\
-                     .format(shutil.which('tmhmm')),
+                help=f"Path to tmhmm, default={shutil.which('tmhmm')} \
+                        (from system lookup)",
                 type=str,
                 default=shutil.which('tmhmm')
         )
         sbp.add_argument(
                 "--targetp",
-                help="Path to targetp, default={} (from system lookup)"\
-                     .format(shutil.which('targetp')),
+                help=f"Path to targetp, default={shutil.which('targetp')} \
+                        (from system lookup)",
                 type=str,
                 default=shutil.which('targetp')
         )
         sbp.add_argument(
                 "--effectorp",
-                help="Path to signalp, default={} (from system lookup)"\
-                     .format(shutil.which('EffectorP')),
+                help=f"Path to signalp, default={shutil.which('EffectorP')} \
+                        (from system lookup)",
                 type=str,
                 default=shutil.which('EffectorP')
         )
@@ -247,7 +247,7 @@ class ArgParse():
 
         sbp = self.subparsers.add_parser(
                 'effector_predictor',
-                help='effector_predictor help'
+                help='Predict effector genes'
         )
         sbp.add_argument(
                 "fasta",
@@ -256,29 +256,29 @@ class ArgParse():
         )
         sbp.add_argument(
                 "--signalp",
-                help="Path to signalp, default={} (from system lookup)"\
-                     .format(shutil.which('signalp')),
+                help=f"Path to signalp, default={shutil.which('signalp')} \
+                        (from system lookup)",
                 type=str,
                 default=shutil.which('signalp')
         )
         sbp.add_argument(
                 "--tmhmm",
-                help="Path to tmhmm, default={} (from system lookup)"\
-                     .format(shutil.which('tmhmm')),
+                help=f"Path to tmhmm, default={shutil.which('tmhmm')} \
+                        (from system lookup)",
                 type=str,
                 default=shutil.which('tmhmm')
         )
         sbp.add_argument(
                 "--targetp",
-                help="Path to targetp, default={} (from system lookup)"\
-                     .format(shutil.which('targetp')),
+                help=f"Path to targetp, default={shutil.which('targetp')} \
+                        (from system lookup)",
                 type=str,
                 default=shutil.which('targetp')
              )
         sbp.add_argument(
                 "--effectorp",
-                help="Path to signalp, default={} (from system lookup)"\
-                     .format(shutil.which('EffectorP')),
+                help=f"Path to signalp, default={shutil.which('EffectorP')} \
+                        (from system lookup)",
                 type=str, default=shutil.which('EffectorP')
         )
         sbp.add_argument(
@@ -315,7 +315,7 @@ class ArgParse():
 
         parser_validate = self.subparsers.add_parser(
                 'validate',
-                help='validate help'
+                help='Validate expected GTF/GFF file formats'
         )
         parser_validate.add_argument(
                 "Gff_genes",
@@ -370,7 +370,7 @@ class ArgParse():
 
         sbp = self.subparsers.add_parser(
                 'isoform_ranking',
-                help='isoform_ranking help'
+                help='Rank isoform based on junction and read coverages'
         )
         sbp.add_argument(
                 "Gff_transcripts",
@@ -455,7 +455,7 @@ class ArgParse():
 
         sbp = self.subparsers.add_parser(
                 'compare',
-                help='compare help'
+                help='Compare gene content between several annotations'
         )
         sbp.add_argument(
                 "fof",
@@ -531,7 +531,7 @@ class ArgParse():
 
         sbp = self.subparsers.add_parser(
                 'select',
-                help='select help'
+                help='Select best gene model'
         )
         sbp.add_argument(
                 "fof",
@@ -728,7 +728,7 @@ class ArgParse():
 
         sbp = self.subparsers.add_parser(
                 'aed',
-                help='aed help'
+                help='Compute AED'
         )
         sbp.add_argument(
                 "Input",
@@ -846,7 +846,7 @@ class ArgParse():
 
         sbp = self.subparsers.add_parser(
                 'filter',
-                help='filter help'
+                help='Filter annotations'
         )
         sbp.add_argument(
                 "Gff_genes",
@@ -899,7 +899,7 @@ class ArgParse():
 
         sbp = self.subparsers.add_parser(
                 'utr_refine',
-                help='utr_refine help'
+                help='Refine UTR boundaries'
         )
         sbp.add_argument(
                 "Gff_genes",
@@ -951,7 +951,7 @@ class ArgParse():
 
         sbp = self.subparsers.add_parser(
                 'aed_compare',
-                help='aed help'
+                help='Compare annotation sets based on their AED scores'
         )
         sbp.add_argument(
                 "fof",
@@ -975,7 +975,7 @@ class ArgParse():
 
         sbp = self.subparsers.add_parser(
                 'aed_strand_annotation_filter',
-                help='aed help'
+                help='Remove opposite overlapping annotations based on AED scores'
         )
         sbp.add_argument(
                 "Input",
@@ -997,7 +997,7 @@ class ArgParse():
 
         sbp = self.subparsers.add_parser(
                 'exonerate_to_gff',
-                help='transform exonerate output to gff format',
+                help='Transform exonerate output to gff format',
                 description="RUN exonerate with:\
                              exonerate --model p2g --showvulgar no \
                              --showalignment no --showquerygff no \
@@ -1033,7 +1033,7 @@ class ArgParse():
 
         sbp = self.subparsers.add_parser(
                 'rename',
-                help='rename gene name with pattern'
+                help='Rename gene name with pattern'
         )
         sbp.add_argument(
                 "Gff_genes",
@@ -1120,6 +1120,41 @@ class ArgParse():
 
         return sbp
 
+    def subparser_curation(self):
+        """Add subparser for Curation cmd"""
+
+        sbp = self.subparsers.add_parser(
+                'curation',
+                help='Define priorities, classify genes for manual \
+                        control or modification'
+        )
+        sbp.add_argument(
+                "Input",
+                help="GFF File with AED tags",
+                type=str
+        )
+        sbp.add_argument(
+                "Output",
+                help="GFF File with new curation tag",
+                type=str
+        )
+        sbp.add_argument(
+                "--graphout",
+                help="output filename of the graph, default=curation.png",
+                type=str,
+                default='curation.png'
+        )
+        sbp.add_argument(
+                "--graphtitle",
+                help="output title of the graph, default=AED categories \
+                      for manual curation",
+                type=str,
+                default='AED categories for manual curation'
+        )
+        sbp.set_defaults(command=cmd.Curation)
+
+        return sbp
+
 
 def main():
     """script entry point"""
@@ -1142,14 +1177,17 @@ def main():
         logging.info("Multi-processs requested: %s procs will be used",
                      cmd.Command.NB_CPUS)
 
+    if 'command' not in args:
+        i.parser.print_help()
+        sys.exit(1)
+
     try:
         command = args.command(args)
         command.run()
-    except Exception as e:
-        print(e)
+    except RuntimeError as error:
+        #print(error)
         i.parser.print_help()
         sys.exit(1)
-#        raise
 
 
 
