@@ -97,7 +97,7 @@ class Graphics():
             bins = np.arange(0.0,1.001,0.001)
             ax_histraw.hist(laed, bins=bins, cumulative=True, histtype="step", color = colors[i], label=src, linewidth=1.5)
             u = ax_histdensity.hist(laed, bins=bins, cumulative=True, histtype="step", density=True, color = colors[i], label=src, linewidth=1.5)
-            logging.info("{} with {} evidences, value at 0.5 AED: {}".format(src, evidence, u[0][500]))
+            logging.info("{} with {} evidence, value at 0.5 AED: {}".format(src, evidence, u[0][500]))
 
         h,l=ax_histraw.get_legend_handles_labels() # get labels and handles from histx  
         ax_legend.legend(h,l, fontsize=20, ncol=ncol, mode="expand", loc="lower center")
@@ -108,11 +108,6 @@ class Graphics():
         canvas.print_figure(out, dpi=80)
 
 
-
-
-
-
-
     @staticmethod
     def plot_distribution(lxs, lys, out="", title="", xax="", yax="",
             color="blue", legend="", grid=None):
@@ -205,21 +200,15 @@ class Graphics():
         ax.set_ylabel("AED with protein evidence", fontsize=20)
         ax.tick_params(labelsize=15)
 
-        #ax.margins(0.02)
-        #ax.set_xlim((0,1))
-        #ax.set_ylim((0,1))
-
         bins = np.arange(0.0,1.01,0.01)
         ax_histx.hist(laed[0], bins=bins, color = '#36953a', edgecolor = 'black', label="AED transcripts")
         ax_histx.vlines(aedtr_filtering, 0, 1, transform=ax_histx.get_xaxis_transform(), colors='r', linestyle="dashed")
         ax_histx.tick_params(labelsize=12)
         ax_histx.set_ylabel("Nb. Transcripts (transcript evidence)", fontsize=20)
-        #ax_histx.legend(fontsize=20)
         ax_histy.hist(laed[1], bins=bins, color = '#fc4b67', edgecolor = 'black', orientation='horizontal', label="AED proteins")
         ax_histy.hlines(aedpr_filtering, 0, 1, transform=ax_histy.get_yaxis_transform(), colors='r', linestyle="dashed")
         ax_histy.tick_params(labelsize=12)
         ax_histy.set_xlabel("Nb. Transcripts (protein evidence)", fontsize=20)
-        #ax_histy.legend(fontsize=20)
 
         h,l=ax_histx.get_legend_handles_labels() # get labels and handles from histx  
         hy,ly=ax_histy.get_legend_handles_labels() # get labels and handles from histy
@@ -237,3 +226,47 @@ class Graphics():
 
         canvas = FigureCanvasAgg(fig)
         canvas.print_figure(out, dpi=80)
+
+
+    @staticmethod
+    def plot_curation_scatter_hist(laed, out="", legend="", title=""):
+        """scatter plot of AEDs with histograms"""
+
+        plt.style.use('bmh')
+
+        fig = plt.Figure(figsize=(20,20))
+        # add gridspec
+        gs = fig.add_gridspec(3,2,width_ratios=(7, 2), height_ratios=(2, 7,2),
+                                      left=0.06, right=0.96, bottom=0.0, top=0.96,
+                                                            wspace=0.1, hspace=0.1)
+        ax = fig.add_subplot(gs[1, 0])
+        ax_histx = fig.add_subplot(gs[0, 0], sharex=ax)
+        ax_histy = fig.add_subplot(gs[1, 1], sharey=ax)
+        ax_legend = fig.add_subplot(gs[2, 0])
+
+        colors=['blue','red','green','maroon','black','orange','purple']
+        for idx,area in enumerate(laed):
+            ax.scatter(area[0],area[1], color=colors[idx], alpha=0.5)
+        # By using ``transform=vax.get_xaxis_transform()`` the y coordinates are scaled
+        ax.set_xlabel("AED with transcript evidence", fontsize=14)
+        ax.set_ylabel("AED with protein evidence", fontsize=14)
+        ax.tick_params(labelsize=15)
+
+        bins = np.arange(0.0,1.01,0.01)
+        colors=['blue','red','green','maroon','black','orange','purple']
+        ax_histx.hist([i[0] for i in laed], bins=bins, color = colors, edgecolor = 'black', label=legend, stacked=True)
+        ax_histx.tick_params(labelsize=12)
+        ax_histx.set_ylabel("Nb. Transcripts (transcript evidence)", fontsize=14)
+        ax_histx.set_title(title,fontsize=22)
+        ax_histy.hist([i[1] for i in laed], bins=bins, color = colors, edgecolor = 'black',orientation='horizontal', label="AED proteins", stacked=True)
+        ax_histy.tick_params(labelsize=12)
+        ax_histy.set_xlabel("Nb. Transcripts (protein evidence)", fontsize=14)
+        h,l=ax_histx.get_legend_handles_labels() # get labels and handles from histx  
+        ax_legend.legend(h,l,fontsize=15, loc='center left')
+        # Hide grid lines
+        ax_legend.grid(False)
+        # change background color
+        ax_legend.set_facecolor('w')
+        ax_legend.axis('off')
+        canvas = FigureCanvasAgg(fig)
+        canvas.print_figure(out, dpi=80)
